/**
 * You can use this hook to throw away intermediate promises, e.g., you have
 * an auto completed input that when typing, loads the autocompletion data
 * dynamically, naturally, you'll only want to use the data of the last typing,
 * all other data should be discarded and not used.
 *
 * ```js
 * const [lastTypingWins] = useLastWins()
 *
 * async function onInput(e) {
 *   try {
 *     const data = await lastTypingWins(getData(e.target.value))
 *     // this promise won, use data.
 *   } catch (err) {
 *     if (err.name === "LastWinsError") {
 *       // this promised failed to win, you probably should ignore it.
 *     }
 *   }
 * }
 * ```
 */
export declare function useLastWins<T>(): readonly [(promise: Promise<T>) => Promise<T>];
