/**
 * Provides mechanisms to cancel promises, also ensures all promises created
 * through this mechanism are canceled when component is unmounted.
 *
 * ```js
 * const [makeCancellable, cancelPromise] = usePromise()
 *
 * useEffect(() => {
 *   ;(async () => {
 *     // promise is automatically canceled if component is unmounted
 *     // before it's resolved.
 *     const dataPromise = makeCancellable(getData())
 *     if (necessary) {
 *       cancelPromise(dataPromise)
 *     }
 *     try {
 *       const data = await dataPromise
 *       // use data
 *     } catch(err) {
 *       if (err.name === "CanceledPromiseError") {
 *         // ignore
 *       }
 *     }
 *   })()
 * }, [])
 * ```
 */
export declare function usePromise(): readonly [<T = any>(promise: Promise<T>) => Promise<T>, <T_1 = any>(promise: Promise<T_1>) => void];
