/**
 * Provides a render function that delays 1 rendering cycle to make
 * rendering consistent with SSR. You can use this it to delay the whole
 * or just part of your rendering. A `rendered` is also provided if you
 * need to know when DOM is actually bound and usable.
 *
 * ```js
 * function MyComp() {
 *   const [compatRender] = useSsrCompat()
 *
 *   return (
 *     <div>
 *       {compatRender(() => (
 *         <div style={{width: `${document.body.clientWidth}px`}} />
 *       ))}
 *     </div>
 *   )
 * }
 * ```
 */
export declare function useSsrCompat(): readonly [(render: () => any) => any, boolean];
