/// <reference types="react" />
/**
 * It works like `useState`, but the state is kept with a ref instead.
 * It is provided to facilitate tracking state changes in timers and
 * other similar situations. It differs from `useRef` in that it does
 * trigger re-render upon set and it can accept a function initializer
 * just like `useState`.
 *
 * ```js
 * const [countRef, setCount] = useStateRef(0)
 *
 * useEffect(() => {
 *   setTimeout(() => {
 *     console.log(countRef.current)
 *   }, 1000)
 *   setCount(v => v + 1) // trigger a re-render
 * }, [])
 * ```
 */
export declare function useStateRef<T>(initializer: T | (() => T)): readonly [import("react").MutableRefObject<T | undefined>, (state: T | ((prevState: T) => T)) => void];
