'use strict';

var React = require('react');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */


var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

function __spreadArray(to, from, pack) {
    if (arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
}

/**
 * This hooks allows you to track a value across renderings.
 * It keeps the value during the last rendering or `undefined` if there
 * is no last rendering.
 *
 * Note you can combine multiple uses of this hook if you want to keep
 * history steps deeper, as you can see in the following example.
 *
 * ```js
 * const prevA = usePrev(a)
 * const prevPrevA = usePrev(prevA)
 * ```
 */
function usePrev(val) {
    var prevRef = React.useRef();
    React.useEffect(function () {
        prevRef.current = val;
    });
    return prevRef.current;
}

/**
 * This component keeps the previous rendered view and its state around
 * so when you switch back to it, no render is required and all of its
 * state are retained. It can optionally retain the scroll offset too.
 *
 * Views are associated and identified by `kpvId`. There are 2 ways to
 * render a view, `kpvComponent` or `kpvRender` function and regardless of
 * what you choose to use, you will receive 2 extra props,
 * `kpvRestored` and `kpvScrollElementRef`. `kpvRestored` tells you whether
 * the view to render is "restored", that is, it was kept as the previous
 * view; `kpvScrollElementRef` in the other hand, allows you to "attach" a
 * scrolling DOM element if you want to persist scroll offset.
 *
 * @example
 * ```jsx
 * let view
 *
 * switch (path) {
 * case "/pages/a":
 *   view = <KeepPrevView kpvId="/pages/a" kpvComponent={CompA} />
 *   break
 * case "/pages/b":
 *   view = <KeepPrevView kpvId="/pages/b" kpvComponent={CompB} />
 *   break
 * }
 *
 * // CompA
 * function CompA({kpvRestored, kpvScrollElementRef}) {
 *   useEffect(() => {
 *     kpvScrollElementRef.current = document.body
 *   }, [])
 *
 *   useEffect(() => {
 *     if (kpvRestored) {
 *       // View is showed up again.
 *     }
 *   }, [kpvRestored])
 *
 *   return ...
 * }
 * ```
 */
var KeepPrevView = React.memo(function KeepPrevView(_a) {
    var kpvId = _a.kpvId, kpvComponent = _a.kpvComponent, kpvRender = _a.kpvRender, others = __rest(_a, ["kpvId", "kpvComponent", "kpvRender"]);
    var prevId = usePrev(kpvId);
    var prevPrevId = usePrev(prevId);
    var restored = kpvId === prevPrevId;
    var scrollElementRef = useScrollRestore$1(restored);
    var Comp = kpvComponent;
    var jsx = kpvRender ? (kpvRender(__assign({ kpvRestored: restored, kpvScrollElementRef: scrollElementRef }, others))) : (React.createElement(Comp, __assign({ kpvRestored: restored, kpvScrollElementRef: scrollElementRef }, others)));
    var prevJsx = usePrev(jsx);
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { key: prevId, style: { display: "none" } }, prevJsx === undefined
            ? undefined
            : React.cloneElement(prevJsx, { kpvRestored: false })),
        React.createElement("div", { key: kpvId }, jsx)));
}, function (prev, next) { return prev.kpvId === next.kpvId; });
function useScrollRestore$1(restored) {
    var _a;
    var scrollElementRef = React.useRef();
    var prevScrollElement = usePrev(scrollElementRef.current);
    var scrollTopRef = React.useRef();
    scrollTopRef.current = (_a = scrollElementRef.current) === null || _a === void 0 ? void 0 : _a.scrollTop;
    var prevScrollTop = usePrev(scrollTopRef.current);
    React.useLayoutEffect(function () {
        if (restored && prevScrollElement != null && prevScrollTop != null) {
            prevScrollElement.scrollTop = prevScrollTop;
        }
    });
    return scrollElementRef;
}

/**
 * This component keeps the specified (`kvId` that starts with `kvPrefix`)
 * rendered view and its state around so when you switch back to it, no
 * render is required and all of its state are retained. It can optionally
 * retain the scroll offset too.
 *
 * Views are associated and identified by `kvId`. There are 2 ways to
 * render a view, `kvComponent` or `kvRender` function and regardless of
 * what you choose to use, you will receive 2 extra props,
 * `kvRestored` and `kvScrollElementRef`. `kvRestored` tells you whether
 * the view to render is "restored", that is, it was kept as the previous
 * view; `kvScrollElementRef` in the other hand, allows you to "attach" a
 * scrolling DOM element if you want to persist scroll offset.
 *
 * @example
 * ```jsx
 * let view
 *
 * switch (path) {
 * case "/pages/a":
 *   view = <KeepView kvPrefix="/pages/a" kvId="/pages/a" kvComponent={CompA} />
 *   break
 * case "/pages/b":
 *   view = <KeepView kvPrefix="/pages/a" kvId="/pages/b" kvComponent={CompB} />
 *   break
 * case "/pages/c":
 *   view = <KeepView kvPrefix="/pages/a" kvId="/pages/c" kvComponent={CompC} />
 *   break
 * }
 *
 * // CompA
 * function CompA({kvRestored, kvScrollElementRef}) {
 *   useEffect(() => {
 *     kvScrollElementRef.current = document.body
 *   }, [])
 *
 *   useEffect(() => {
 *     if (kvRestored) {
 *       // View is showed up again.
 *     }
 *   }, [kvRestored])
 *
 *   return ...
 * }
 * ```
 */
var KeepView = React.memo(function KeepView(_a) {
    var kvId = _a.kvId, kvPrefix = _a.kvPrefix, kvComponent = _a.kvComponent, kvRender = _a.kvRender, others = __rest(_a, ["kvId", "kvPrefix", "kvComponent", "kvRender"]);
    var keptId = React.useRef();
    var keptJsx = React.useRef();
    var prevId = usePrev(kvId);
    var restored = kvId === keptId.current;
    if (restored) {
        keptId.current = undefined;
        keptJsx.current = undefined;
    }
    var shallKeep = !!(prevId === null || prevId === void 0 ? void 0 : prevId.startsWith(kvPrefix));
    var scrollElementRef = useScrollRestore(shallKeep, restored);
    var Comp = kvComponent;
    var jsx = kvRender ? (kvRender(__assign({ kvRestored: restored, kvScrollElementRef: scrollElementRef }, others))) : (React.createElement(Comp, __assign({ kvRestored: restored, kvScrollElementRef: scrollElementRef }, others)));
    var prevJsx = usePrev(jsx);
    if (shallKeep) {
        keptId.current = prevId;
        keptJsx.current = prevJsx;
    }
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { key: keptId.current, style: { display: "none" } }, keptJsx.current === undefined
            ? undefined
            : React.cloneElement(keptJsx.current, { kvRestored: false })),
        React.createElement("div", { key: kvId }, jsx)));
}, function (prev, next) { return prev.kvId === next.kvId; });
function useScrollRestore(shallKeep, restored) {
    var _a;
    var scrollElementRef = React.useRef();
    var scrollTopRef = React.useRef();
    var prevScrollTop = usePrev((_a = scrollElementRef.current) === null || _a === void 0 ? void 0 : _a.scrollTop);
    if (shallKeep) {
        scrollTopRef.current = prevScrollTop;
    }
    React.useLayoutEffect(function () {
        if (restored &&
            scrollElementRef.current != null &&
            scrollTopRef.current != null) {
            scrollElementRef.current.scrollTop = scrollTopRef.current;
        }
    });
    return scrollElementRef;
}

/**
 * Join your classes for a component, any non-string values will be excluded.
 *
 * ```js
 * const level = 1
 * const visible = false
 * let dynamicClassName
 *
 * <MyComp
 *   className={cls(
 *     "container",
 *     `level-${level}`,
 *     visible && "visible",
 *     dynamicClassName,
 *   )}
 * />
 *
 * // Will be <MyComp className="container level-1" />
 * ```
 */
function cls() {
    var classes = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        classes[_i] = arguments[_i];
    }
    return (classes.filter(function (c) { return typeof c === "string" && c !== ""; }).join(" ") ||
        undefined);
}

/**
 * Handles the composition events for IME input and provides an unified
 * callback.
 */
function useCompositionChange(callback) {
    var isComposing = React.useRef(false);
    var onChange = React.useCallback(function (e) {
        var others = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            others[_i - 1] = arguments[_i];
        }
        if (isComposing.current)
            return;
        callback.apply(void 0, __spreadArray([e], others, false));
    }, [callback]);
    var onCompositionStart = React.useCallback(function (_e) {
        isComposing.current = true;
    }, []);
    var onCompositionEnd = React.useCallback(function (e) {
        var others = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            others[_i - 1] = arguments[_i];
        }
        isComposing.current = false;
        if (!e.data)
            return;
        callback.apply(void 0, __spreadArray([e], others, false));
    }, [callback]);
    return { onChange: onChange, onCompositionStart: onCompositionStart, onCompositionEnd: onCompositionEnd };
}

/**
 * Creates a state and keeps it updated everytime its deps changes, however
 * this update happens 1 cycle later than its deps. If this late update is not
 * what you're looking for, `useMemo` or a simple expression is what you need.
 */
function useDependentState(updater, deps, initialState) {
    var _a = React.useState(initialState !== null && initialState !== void 0 ? initialState : updater), value = _a[0], setValue = _a[1];
    React.useEffect(function () { return setValue(updater()); }, deps);
    return [value, setValue];
}

/**
 * Gives you a function that when called, causes a re-render.
 */
function useForceUpdate() {
    var _a = React.useState(), setState = _a[1];
    var forceUpdate = React.useCallback(function () { return setState({}); }, []);
    return forceUpdate;
}

/**
 * Implements a key long press event behavior.
 */
function useKeyLongPress(key, duration, callback) {
    var timer = React.useRef();
    var onKeyDown = React.useCallback(function (e) {
        if (e.key !== key || timer.current != null)
            return;
        timer.current = setTimeout(callback, duration);
    }, [key, duration, callback]);
    var onKeyUp = React.useCallback(function (e) {
        if (e.key !== key)
            return;
        clearTimeout(timer.current);
        timer.current = null;
    }, [key]);
    return { onKeyDown: onKeyDown, onKeyUp: onKeyUp };
}

/**
 * You can use this hook to throw away intermediate promises, e.g., you have
 * an auto completed input that when typing, loads the autocompletion data
 * dynamically, naturally, you'll only want to use the data of the last typing,
 * all other data should be discarded and not used.
 *
 * ```js
 * const [lastTypingWins] = useLastWins()
 *
 * async function onInput(e) {
 *   try {
 *     const data = await lastTypingWins(getData(e.target.value))
 *     // this promise won, use data.
 *   } catch (err) {
 *     if (err.name === "LastWinsError") {
 *       // this promised failed to win, you probably should ignore it.
 *     }
 *   }
 * }
 * ```
 */
function useLastWins() {
    var _this = this;
    var lastPromise = React.useRef();
    var lastWins = React.useCallback(function (promise) { return __awaiter(_this, void 0, void 0, function () {
        var ret, err;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    lastPromise.current = promise;
                    return [4 /*yield*/, promise];
                case 1:
                    ret = _a.sent();
                    if (lastPromise.current === promise) {
                        return [2 /*return*/, ret];
                    }
                    else {
                        err = new Error("Not the last");
                        err.name = "LastWinsError";
                        throw err;
                    }
            }
        });
    }); }, []);
    return [lastWins];
}

/**
 * Giving a function that returns a value or promise, this hook provides
 * some extra metadata for users to use. Metadata is only computed if you
 * used it, therefore, any not used metadata will not affect your component's
 * rendering.
 */
function useMeta(fn, deps) {
    var _this = this;
    var _a = React.useState(true), _loading = _a[0], setLoading = _a[1];
    var _b = React.useState(), _data = _b[0], setData = _b[1];
    var _c = React.useState(), _error = _c[0], setError = _c[1];
    var state = React.useRef({
        usedLoading: false,
        usedData: false,
        usedError: false,
    });
    var refetch = React.useCallback(function () { return __awaiter(_this, void 0, void 0, function () {
        var ret, err_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (state.current.usedLoading) {
                        setLoading(true);
                    }
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, fn()];
                case 2:
                    ret = _a.sent();
                    if (state.current.usedLoading) {
                        setLoading(false);
                    }
                    if (state.current.usedData) {
                        setData(ret);
                    }
                    if (state.current.usedError) {
                        setError(undefined);
                    }
                    return [3 /*break*/, 4];
                case 3:
                    err_1 = _a.sent();
                    if (state.current.usedLoading) {
                        setLoading(false);
                    }
                    if (state.current.usedData) {
                        setData(undefined);
                    }
                    if (state.current.usedError) {
                        setError(err_1);
                    }
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/];
            }
        });
    }); }, [fn]);
    React.useEffect(function () {
        refetch();
    }, deps);
    return {
        get loading() {
            state.current.usedLoading = true;
            return _loading;
        },
        get data() {
            state.current.usedData = true;
            return _data;
        },
        get error() {
            state.current.usedError = true;
            return _error;
        },
        refetch: refetch,
    };
}

function makeCancellableImpl(promise, callback) {
    var isCanceled = false;
    return {
        promise: new Promise(function (resolve, reject) {
            promise.then(function (val) {
                callback(promise);
                if (isCanceled) {
                    var err = new Error("canceled");
                    err.name = "CanceledPromiseError";
                    reject(err);
                }
                else {
                    resolve(val);
                }
            }, function (err) {
                callback(promise);
                if (isCanceled) {
                    var err_1 = new Error("canceled");
                    err_1.name = "CanceledPromiseError";
                    reject(err_1);
                }
                else {
                    reject(err);
                }
            });
        }),
        cancel: function () {
            isCanceled = true;
        },
    };
}
/**
 * Provides mechanisms to cancel promises, also ensures all promises created
 * through this mechanism are canceled when component is unmounted.
 *
 * ```js
 * const [makeCancellable, cancelPromise] = usePromise()
 *
 * useEffect(() => {
 *   ;(async () => {
 *     // promise is automatically canceled if component is unmounted
 *     // before it's resolved.
 *     const dataPromise = makeCancellable(getData())
 *     if (necessary) {
 *       cancelPromise(dataPromise)
 *     }
 *     try {
 *       const data = await dataPromise
 *       // use data
 *     } catch(err) {
 *       if (err.name === "CanceledPromiseError") {
 *         // ignore
 *       }
 *     }
 *   })()
 * }, [])
 * ```
 */
function usePromise() {
    var promisesRef = React.useRef([]);
    var promises = promisesRef.current;
    var removePromise = React.useCallback(function (promise) {
        var index = promises.findIndex(function (p) { return p.promise === promise; });
        if (index >= 0) {
            promises.splice(index, 1);
        }
    }, [promises]);
    var makeCancellable = React.useCallback(function (promise) {
        var cancellable = makeCancellableImpl(promise, removePromise);
        promises.push(cancellable);
        return cancellable.promise;
    }, [promises]);
    var cancelPromise = React.useCallback(function (promise) {
        for (var _i = 0, promises_1 = promises; _i < promises_1.length; _i++) {
            var p = promises_1[_i];
            if (p.promise === promise) {
                p.cancel();
                break;
            }
        }
    }, [promises]);
    React.useEffect(function () {
        return function () {
            for (var _i = 0, promises_2 = promises; _i < promises_2.length; _i++) {
                var p = promises_2[_i];
                p.cancel();
            }
        };
    }, []);
    return [makeCancellable, cancelPromise];
}

/**
 * Provides a render function that delays 1 rendering cycle to make
 * rendering consistent with SSR. You can use this it to delay the whole
 * or just part of your rendering. A `rendered` is also provided if you
 * need to know when DOM is actually bound and usable.
 *
 * ```js
 * function MyComp() {
 *   const [compatRender] = useSsrCompat()
 *
 *   return (
 *     <div>
 *       {compatRender(() => (
 *         <div style={{width: `${document.body.clientWidth}px`}} />
 *       ))}
 *     </div>
 *   )
 * }
 * ```
 */
function useSsrCompat() {
    var _a = React.useState(false), mounted = _a[0], setMounted = _a[1];
    var rendered = useDependentState(function () { return mounted; }, [mounted])[0];
    React.useEffect(function () {
        setMounted(true);
    }, []);
    return [
        React.useCallback(function (render) { return (mounted ? render() : null); }, [mounted]),
        rendered,
    ];
}

/**
 * It works like `useState`, but the state is kept with a ref instead.
 * It is provided to facilitate tracking state changes in timers and
 * other similar situations. It differs from `useRef` in that it does
 * trigger re-render upon set and it can accept a function initializer
 * just like `useState`.
 *
 * ```js
 * const [countRef, setCount] = useStateRef(0)
 *
 * useEffect(() => {
 *   setTimeout(() => {
 *     console.log(countRef.current)
 *   }, 1000)
 *   setCount(v => v + 1) // trigger a re-render
 * }, [])
 * ```
 */
function useStateRef(initializer) {
    var stateRef = React.useRef();
    var forceUpdate = useForceUpdate();
    if (stateRef.current === undefined) {
        if (isInitFunc(initializer)) {
            stateRef.current = initializer();
        }
        else {
            stateRef.current = initializer;
        }
    }
    function setState(state) {
        if (isUpdaterFunc(state)) {
            var updater = state;
            stateRef.current = updater(stateRef.current);
        }
        else {
            stateRef.current = state;
        }
        forceUpdate();
    }
    return [stateRef, setState];
}
function isInitFunc(x) {
    return typeof x === "function";
}
function isUpdaterFunc(state) {
    return typeof state === "function";
}

function getGlobal() {
    var g = undefined;
    try {
        g = window;
        return g;
    }
    catch (e) {
        // ignore, try next
    }
    try {
        g = global;
        return g;
    }
    catch (e) {
        // ignore, try next
    }
    return g;
}
var win = getGlobal();
/**
 * Provide special versions of the different timers so they're
 * cleared/canceled upon component unmount automatically.
 */
function useTimers() {
    var idsRef = React.useRef({});
    var methodsRef = React.useRef({});
    var ids = idsRef.current;
    var methods = methodsRef.current;
    if (methods.setTimeout == null) {
        [
            ["setTimeout", "clearTimeout"],
            ["setInterval", "clearInterval"],
            ["setImmediate", "clearImmediate"],
            ["requestAnimationFrame", "cancelAnimationFrame"],
        ].forEach(function (_a) {
            var f = _a[0], cf = _a[1];
            methods[f] = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                var id = win[f].apply(win, args);
                if (!ids[f]) {
                    ids[f] = new Set();
                }
                ids[f].add(id);
                return id;
            };
            methods[cf] = function (id) {
                var _a;
                win[cf](id);
                (_a = ids[f]) === null || _a === void 0 ? void 0 : _a.delete(id);
            };
        });
    }
    React.useEffect(function () {
        return function () {
            [
                ["setTimeout", "clearTimeout"],
                ["setInterval", "clearInterval"],
                ["setImmediate", "clearImmediate"],
                ["requestAnimationFrame", "cancelAnimationFrame"],
            ].forEach(function (_a) {
                var _b;
                var f = _a[0], cf = _a[1];
                (_b = ids[f]) === null || _b === void 0 ? void 0 : _b.forEach(function (id) { return win[cf](id); });
            });
        };
    }, []);
    return methods;
}

/**
 * Avoids an action being called multiple times, that is, it allows one
 * execution at a time. It optionally provides a `duringAction` to indicate
 * whether execution is finished or not.
 *
 * ```js
 * const { action, duringAction } = useWaitedAction(async (e) => {
 *   // ...
 * })
 *
 * return (
 *   <button onClick={action} disabled={duringAction}>Button</button>
 * )
 * ```
 */
function useWaitedAction(fn) {
    var _this = this;
    var _a = React.useState(false), _duringAction = _a[0], setDuringAction = _a[1];
    var state = React.useRef({
        calling: false,
        usedDuringAction: false,
    });
    var action = React.useCallback(function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!!state.current.calling) return [3 /*break*/, 4];
                        state.current.calling = true;
                        if (state.current.usedDuringAction) {
                            setDuringAction(true);
                        }
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, , 3, 4]);
                        return [4 /*yield*/, (fn === null || fn === void 0 ? void 0 : fn.apply(void 0, args))];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        if (state.current.usedDuringAction) {
                            setDuringAction(false);
                        }
                        state.current.calling = false;
                        return [7 /*endfinally*/];
                    case 4: return [2 /*return*/];
                }
            });
        });
    }, [fn]);
    if (fn == null)
        return { action: undefined, duringAction: false };
    return {
        action: action,
        get duringAction() {
            state.current.usedDuringAction = true;
            return _duringAction;
        },
    };
}

exports.KeepPrevView = KeepPrevView;
exports.KeepView = KeepView;
exports.cls = cls;
exports.useCompositionChange = useCompositionChange;
exports.useDependentState = useDependentState;
exports.useForceUpdate = useForceUpdate;
exports.useKeyLongPress = useKeyLongPress;
exports.useLastWins = useLastWins;
exports.useMeta = useMeta;
exports.usePrev = usePrev;
exports.usePromise = usePromise;
exports.useSsrCompat = useSsrCompat;
exports.useStateRef = useStateRef;
exports.useTimers = useTimers;
exports.useWaitedAction = useWaitedAction;
